#include <p18cxxx.h>
#include <delays.h>

#define		DelayUs(x) { unsigned char _dcnt; \
				_dcnt = (x)*(16/12); \
				while(--_dcnt != 0) \
					continue; }

// using pin RD7 for 1-wire interface
#define BAT_PIN PORTDbits.RD7
#define BAT_TRI TRISDbits.TRISD7

typedef struct bitvar{
	unsigned Bit:1;
} bitvar;

bitvar batPresence_flag, batShortCircuit_flag, batError_flag, batData;

//bitvar Test
//Test.Bit = 0;
//Test.Bit = 1;

void batReset(void);
char batRead(void);
void batWrite(char);
int getBatStat(void);

int getBatStat(void) {

	char dataReadLow, dataReadHigh;
	int completeData = 0;

	// disable interrupts
	INTCONbits.GIEH = 0;
	INTCONbits.GIEL = 0;

	// initialize connection
	batReset();

	// send "Skip Meet Address" command (0xcc)
	batWrite(0xcc);

	// request data in Currest Register MSB
	batWrite(0x10);
	dataReadHigh = batRead();
	batWrite(0x11);
	dataReadLow = batRead();

	completeData = (int)dataReadHigh;	
	//completeData = completeData << 8;
	completeData *= 256;
	completeData += (int)dataReadLow;

	// enable interrupts
	INTCONbits.GIEH = 1;
	INTCONbits.GIEL = 1;

	return completeData;
}

void batReset(void) {
//	char count;
	batError_flag.Bit = 0;
	batPresence_flag.Bit = 0;
	
	// high-z state
	BAT_TRI = 1;	

	BAT_PIN = 0;
	BAT_TRI = 0; // output 0 on battery status line

	DelayUs(500);

	BAT_TRI = 1; // release battery status line
	DelayUs(70);

	batPresence_flag.Bit = BAT_PIN;

	DelayUs(430);

	//BAT_TRI = 1;
//	BAT_PIN = 0;
//
//	BAT_TRI = 0;
//	for(count=47; count>0; count--) {
//		DelayUs(10);
//	}
//	BAT_TRI = 1;
//
//	DelayUs(50);
//	DelayUs(10);
//	batPresence_flag.Bit = BAT_PIN;
//	
//	for (count=41;count>0;count--) {
//		DelayUs(10);
//	}
//	batShortCircuit_flag.Bit = BAT_PIN;
//
//	if (batShortCircuit_flag.Bit == 0) {
//		batError_flag.Bit = 1;
//	}
//
//	if (batPresence_flag.Bit == 1) {
//		batError_flag.Bit = 1;
//	}
//
//	DelayUs(1);
}

void batWrite(char data) {

	char count;

	for (count=8; count>0; count--) {
		BAT_PIN = 0;
		BAT_TRI = 0;
		DelayUs(3);
		BAT_TRI = data&0x01;
		DelayUs(60);

		BAT_TRI = 1;
		DelayUs(2);
		data = data >> 1;
	}

//	char count;
//	for (count=8; count>0; count--) {
//		BAT_PIN = 0;
//		BAT_TRI = 0;
//		DelayUs(5);
//		BAT_TRI = batData.Bit;
//		DelayUs(50);
//		DelayUs(5);
//
//		BAT_TRI = 1;
//
//		DelayUs(1);
//		
//		data = data >> 1;
//	}
//	DelayUs(1);
}

char batRead(void) {
	char count;
	char data = 0;

	for(count=8; count>0; count--) {
		BAT_PIN = 0;
		BAT_TRI = 0;
		DelayUs(6);
		BAT_TRI = 1;
		DelayUs(4);
		batData.Bit = BAT_PIN;
		DelayUs(50);

		data = data >> 1;
		if (batData.Bit == 1) {
			data += 0x80;
		}
		DelayUs(1);
	}

//	for (count=8; count>0; count--) {
//		BAT_PIN = 0;
//		BAT_TRI = 0;
//		DelayUs(5);
//		BAT_TRI = 1;
//		DelayUs(5);
//		batData.Bit = BAT_PIN;
//		DelayUs(50);
//
//		data = data >> 1;
//		if (batData.Bit == 1) {
//			data += 0x80;
//		}
//		DelayUs(1);
//	}
//	DelayUs(1);
//	return data;
}